// ARIsample.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "ariadne.h"
#include <stdio.h>


//returns number of local variables in a function
DWORD GetVariablesCount( AIRFactoryHandle hFactory, const struct SAIRInstruction* function )
{
    DWORD dwVariables = 0;
    struct SAIRMc local_mcs, mc;

    //get function's local storage
    local_mcs.m_id = function->m_args[1].m_corr_id;
    if( AirGetMc( hFactory, &local_mcs ) != eAIRErrNoErr )
        return 0;

    ///simply enumerate all the local variables in a function and count them
    if( AirGetFirstMc( hFactory, &local_mcs, &mc ) != eAIRErrNoErr )
        return 0;
    do ++dwVariables;
    while ( AirGetNextMc( hFactory, &mc, &mc ) == eAIRErrNoErr );

    return dwVariables;
}

//returns number of basic blocks in a factory
DWORD GetBasicBlocksCount( AIRFactoryHandle hFactory )
{
    DWORD dwBasicBlocks;
    AIRBbseHandle bbse;

    //we have only one function so for simplicity just get total count of basic blocks
    if( AirCreateBbse( hFactory, &bbse ) != eAIRErrNoErr )
        return 0;
    if( AirDistinguishBasicBlocks( bbse ) != eAIRErrNoErr )
    {
        AirFreeBbse( bbse );
        return 0;
    }
    dwBasicBlocks = AirGetBbCount( bbse );
    AirFreeBbse( bbse );

    return dwBasicBlocks;
}

//returns number of instructions in a function
DWORD GetInstructionsCount( AIRFactoryHandle hFactory, const struct SAIRInstruction* function )
{
    DWORD dwInstructions = 0;
    struct SAIRInstruction instr;

    //simply enumerate all the instructions in a function and count them
    if( AirGetFirstInstr( hFactory, function, &instr ) != eAIRErrNoErr )
        return 0;
    do ++dwInstructions;
    while ( AirGetNextInstr( hFactory, &instr, &instr ) == eAIRErrNoErr );
    return dwInstructions;
}

void main()
{
    AIDHANDLE hAIDisasm;
    AIRFactoryHandle hFactory;
    AIRCtxHandle hContext;
    struct SAIRInstruction function;

    //--------------------------------------------------------------------------------
    //at first open our file and disassemble it
    //--------------------------------------------------------------------------------
	printf("AidCreateEngine...\n");
    //create AIDisasm instance
    hAIDisasm = AidCreateEngine();

	printf("AidLoadPeFile...");
    //try to open protected2.exe file from samples
    if( !AidLoadPeFile( hAIDisasm, L"TestAri.exe" ) )
    {
        //can't open PE file
        //free AIDisasm memory
		printf("can't open PE file\n");
        AidFreeEngine( hAIDisasm );
        return;
    }
	printf("\nAidDisassemble...");
    //disassemble file with default parameters
    if( !AidDisassemble( hAIDisasm, AID_DEFAULT_DISASM_PARAMS ) )
    {
        //something went wrong while disassembling
        //no need no call AidCloseFile, AidFreeEngine calls it
        printf("something went wrong while disassembling");
		AidFreeEngine( hAIDisasm );
        return;
    }
    //--------------------------------------------------------------------------------
    //file is disassembled and now we're ready to generate AIR code
    //--------------------------------------------------------------------------------
	printf("\nAirCreateFactory...");
    //this is x86 PE file
    if( AirCreateFactory( eAIRx86, &hFactory ) != eAIRErrNoErr )
    {
        printf("can't create factory");
		//can't create factory
        //no need no call AidCloseFile, AidFreeEngine calls it
        AidFreeEngine( hAIDisasm );
        return;
    }

    //create emulator context, don't run process so filename and rva don't matter
    //set initial values (for registers, return value from ESP, etc.) to default
    printf("\nAirCreateCtx...");
	if( AirCreateCtx( hFactory, &hContext, FALSE, 0, 0 ) != eAIRErrNoErr )
    {
        printf("can't create emulator context");
		//can't create emulator context
        //free AIR factory memory
        AirFreeFactory( hFactory );
        //no need no call AidCloseFile, AidFreeEngine calls it
        AidFreeEngine( hAIDisasm );
        return;
    }

    //generate AIR code by tracing from RVA 0x1160 to RVA 0x1165. there is no virtual machine here
    //so internal context VA-s are set to 0
    printf("\nAirGenerateTrace...");
	if( AirGenerateTrace( hFactory, hAIDisasm, hContext, 0x1010, 0x1025, 0, 0 ) != eAIRErrNoErr )
    {
        //something went wrong while generating AIR code from PE file
        //free emulator context
		printf("something went wrong while generating AIR code from PE file");
        AirFreeCtx( hContext );
        //free AIR factory memory
        AirFreeFactory( hFactory );
        //no need no call AidCloseFile, AidFreeEngine calls it
        AidFreeEngine( hAIDisasm );
        return;
    }

    //compile generated code to adjust links between instructions
    printf("\nAirCompile...");
	if( AirCompile( hFactory ) != eAIRErrNoErr )
    {
        printf("something went wrong while compiling AIR code");
		//something went wrong while compiling AIR code
        //free emulator context
        AirFreeCtx( hContext );
        //free AIR factory memory
        AirFreeFactory( hFactory );
        //no need no call AidCloseFile, AidFreeEngine calls it
        AidFreeEngine( hAIDisasm );
        return;
    }

    //fill ID for our function in factory
    //we're looking for function so Function Family
    function.m_id.m_family = eAIRFamFunc;
    //level is default
    function.m_id.m_level = 0;
    //set id to RVA of function, we started tracing from 0x1160
    function.m_id.m_id = 0x1010;
    //id1 is not initialized
    function.m_id.m_id1 = 0xffffffff;
    //id2 is not initialized
    function.m_id.m_id2 = 0xffffffff;
	printf("\nAirGetInstr...");
    if( AirGetInstr( hFactory, &function ) != eAIRErrNoErr )
    {
        printf("can't find function");
		//can't find function
        //free emulator context
        AirFreeCtx( hContext );
        //free AIR factory memory
        AirFreeFactory( hFactory );
        //no need no call AidCloseFile, AidFreeEngine calls it
        AidFreeEngine( hAIDisasm );
        return;
    }

    //print results of generation
    printf( "before:\n");
    printf( "    instructions: %u\n", GetInstructionsCount( hFactory, &function ) );
    printf( "    basic blocks: %u\n", GetBasicBlocksCount( hFactory ) );
    printf( "    local variables: %u\n", GetVariablesCount( hFactory, &function ) );

    //save generated AIR code to text file
    if( AirSaveToText( hFactory, L"before2.txt" ) != eAIRErrNoErr )
    {
        //can't save AIR code to text file
        //free emulator context
        AirFreeCtx( hContext );
        //free AIR factory memory
        AirFreeFactory( hFactory );
        //no need no call AidCloseFile, AidFreeEngine calls it
        AidFreeEngine( hAIDisasm );
        return;
    }

    //--------------------------------------------------------------------------------
    //now we have AIR code, it's time to optimize it and save in text for the user
    //--------------------------------------------------------------------------------
	printf("\nAirOptimize...");
    //optimize AIR code with static and dynamic methods
    if( AirOptimize ( hFactory, hAIDisasm, hContext, eAIRStatic | eAIRDynamic ) != eAIRErrNoErr )
    {
        printf("omething went wrong in optimization of AIR code");
		//something went wrong in optimization of AIR code
        //free emulator context
        AirFreeCtx( hContext );
        //free AIR factory memory
        AirFreeFactory( hFactory );
        //no need no call AidCloseFile, AidFreeEngine calls it
        AidFreeEngine( hAIDisasm );
        return;
    }

    //we don't need emulator context anymore so free it
    AirFreeCtx( hContext );

    //print results of optimization
    printf( "after:\n");
    printf( "    instructions: %u\n", GetInstructionsCount( hFactory, &function ) );
    printf( "    basic blocks: %u\n", GetBasicBlocksCount( hFactory ) );
    printf( "    local variables: %u\n", GetVariablesCount( hFactory, &function ) );

    //save optimized AIR code to text file
    if( AirSaveToText( hFactory, L"after2.txt" ) != eAIRErrNoErr )
    {
        //can't save AIR code to text file
        //free AIR factory memory
        AirFreeFactory( hFactory );
        //no need no call AidCloseFile, AidFreeEngine calls it
        AidFreeEngine( hAIDisasm );
        return;
    }

    //--------------------------------------------------------------------------------
    //our work is done, free all the allocated memory
    //--------------------------------------------------------------------------------

    //free AIR factory memory
    AirFreeFactory( hFactory );

    //actually there is no need no call AidCloseFile, AidFreeEngine calls it
    //but for the sake of variety let's do it the different way
    AidCloseFile( hAIDisasm );

    //free AIDisasm memory
    AidFreeEngine( hAIDisasm );
}

